

## OPTION 1 - Any

* the most flexible on schema side- aby object can be placed insife `Any` container
* client is using predefined objects/messages - all attributes defined
* no tooling to deserialise to/from json - investigation per language/platform required plus additional implementation efforts. Would be a case anytime we introduce new tools etc
* Kafka UI - won't deserialize object
* Parquet / sql - object not usable/queryable



## OPTION 2 - OneOf

* no flexibility on client side - all attributes predefined
* every type on `OneOf` is individual field in main protobuf message - that means eche of the has unique attribute name and index. No so generic for sql - problem with  querying same attribute across events - problem wi. 

```
  oneof test_oneof {
    tunein.clarity.ueapi.events.v1.ListeningBaseEvent listening_event = 50;
    tunein.clarity.ueapi.events.v1.AdsPrerollEligibilityEvent ads_preroll_eligibility_event = 51;
  }
```

* messages fully typed
* no issues with kafka UI deserialisation & querying parquet



Sample:
```
{
	"deviceId": "device-xxx",
	"messageId": "06be6243-de5c-4e0c-abb4-4985ff0b8475",
	"eventTs": "1970-01-01T00:00:00Z",
    "sentAt":"1970-01-01T00:00:00Z",
   "receivedAt":"1970-01-01T00:00:00Z"
	"type": "EVENT_TYPE_UNSPECIFIED",
	"event": "EVENT_CODE_UNSPECIFIED",
	"properties": {},
	"listeningEvent": {// predefined proto message used in OneOf
		"listeningInfo": {
			"listenId": "listenId",
			"guideId": "gude123",
			"parentGuideId": "parent"
		},
        "string":"string"
	},
   "context":{

   }
}

```

## OPTION 3 - flexible map<string, string>
* full flexibility on client side to provide event details attributes
* typing is lost - additional efforts on real time processing, less clean code
* no issues with kafka UI deserialisation & querying parquet

Sample:
```
{
	"deviceId": "device-xxx",
	"messageId": "0a6904f9-07b6-4164-8c27-46a5e1cadfad",
	"eventTs": "1970-01-01T00:00:00Z",
    "sentAt":"1970-01-01T00:00:00Z",
   "receivedAt":"1970-01-01T00:00:00Z"
	"type": "EVENT_TYPE_UNSPECIFIED",
	"event": "EVENT_CODE_UNSPECIFIED",
	"properties": {
        "string":"string",
        "string":"string",
        "string":"string"
    },
   "context":{

   }
}

```

## OPTION 4 - predefined object with addiional flexible map<string, string> attribute
* predefining most of the attributes
* keeping flexibility to send additional attributes
* better separation between `event`,  `context` and message related attributes (e.g. `setAt`, `receivedAt`)
* typing lost for unique per event attributes
* no issues with kafka UI deserialisation & querying parquet

```
{
   "deviceId":"device-xxx",
   "messageId":"06be6243-de5c-4e0c-abb4-4985ff0b8475",
   "sentAt":"1970-01-01T00:00:00Z",
   "receivedAt":"1970-01-01T00:00:00Z"
   "eventDetails":{ // generic proto message 
      "type":"EVENT_TYPE_UNSPECIFIED",
      "event":"EVENT_CODE_UNSPECIFIED",
      "eventTs":"1970-01-01T00:00:00Z", // event timestamp

      "listeningInfo":{ // common object used in most of events
         "listenId":"listenId",
         "guideId":"gude123",
         "parentGuideId":"parent"
      },
      	"properties": { // map<string, string> for additional attributes unique to the event
            "string":"string",
            "string":"string",
            "string":"string"
    }
   },
   "context":{

   }
}
```


## OPTION 5 - predefined object with addiional OneOf (for unique event attributes)
* predefining all attributes
* better separation between `event`,  `context` and message related attributes (e.g. `setAt`, `receivedAt`)
* fully typed
* no issues with kafka UI deserialisation & querying parquet
* in `oneOf` messages only unique attributes used - querying common attributes with no issue, having same attribute names in multiple events details messages should be limitted.


```

{
   "deviceId":"device-xxx",
   "messageId":"06be6243-de5c-4e0c-abb4-4985ff0b8475",
   "sentAt":"1970-01-01T00:00:00Z",
   "receivedAt":"1970-01-01T00:00:00Z"
   "eventDetails":{ // generic proto message 
      "type":"EVENT_TYPE_UNSPECIFIED",
      "event":"EVENT_CODE_UNSPECIFIED",
      "eventTs":"1970-01-01T00:00:00Z", // event timestamp

      "listeningInfo":{ // common object used in most of events
         "listenId":"listenId",
         "guideId":"gude123",
         "parentGuideId":"parent"
      },

      "listeningEventDetails":{ // predefined proto message used in OneOf
         "string":"string",
         "bool":true,
         "int":3
      }
   },
   "context":{

   }
}
```

## OPTION 6 - protobuf message per event
* message per events
    * client use individual messages per event and serialize to json
    * downstream process (validation process) put that into single 'event' type
        * 'event' have all attributes combined

* predefining attributes per event
* typing in place
* generic SerDe with no issues (kafka UI / parquet)
* schema code duplication 
* adding new events required adding attributes to common 'Event' message


```
{
  "deviceId": "device-xxx",
  "messageId": "0a6904f9-07b6-4164-8c27-46a5e1cadfad",
  "eventTs": "1970-01-01T00:00:00Z",
  "sentAt": "1970-01-01T00:00:00Z",
  "receivedAt": "1970-01-01T00:00:00Z",
  "type": "EVENT_TYPE_UNSPECIFIED",
  "event": "EVENT_CODE_UNSPECIFIED",
  "context": {

  },

  "listenId": "listenId",
  "guideId": "gude123",
  "parentGuideId": "parent",
  "string": "string",
  "bool": true,
  "int": 3
}

```


